#!/usr/bin/python
# -*- coding:utf-8 -*-

from dom import *

import os, sys, string, re, time, types

class EarthQuakeInformation:
    
    def __init__( self ):
        self.url           = None
        self.url_prev      = None
        self.url_post      = None
        self.longitude     = 0.
        self.latitude      = 0.
        self.magnitude     = 0.
        self.depth         = 0
        self.max_intensity = 0
        self.sorted        = True
        lt = time.localtime()
        self.report_date   = ( lt[ 0 ] + 1900, lt[ 1 ], lt[ 2 ], lt[ 3 ], lt[ 4 ] )
        self.incident_date = self.report_date
        self.arealist      = []
        self.qlist         = {}
        pass

    def is_linked( self ):
        if self.url_prev and self.url_post and self.url: return True
        return False

    def load_from_file( self, filename ):
        for line in open( filename ).readlines():
            m = re.match( "(\\w+)\\s*:\\s*(.*)", line.strip() )
            if m == None: continue
            key, value = m.group( 1 ), m.group( 2 )
            if key == "URL"        : self.url = value
            elif key == "Previous" : self.url_prev = value
            elif key == "Next"     : self.url_post = value
            elif key == "Report"   :
                items = re.split( "\\s*/\\s*", value )
                self.set_report_date( items[ 0 ], items[ 1 ], items[ 2 ], items[ 3 ], items[ 4 ] )
            elif key == "Incident":
                items = re.split( "\\s*/\\s*", value )
                self.set_date( items[ 2 ], items[ 3 ], items[ 4 ] )
            elif key == "Latitude": self.latitude = string.atof( value )
            elif key == "Longitude":self.longitude = string.atof( value )
            elif key == "Magnitude":self.magnitude = string.atof( value )
            elif key == "Depth":self.depth = string.atoi( value )
            elif re.match( "\\d+", key ):
                intensity = string.atoi( value )
                if intensity > self.max_intensity:
                    self.max_intensity = intensity
                self.qlist[ key ] = [ 1, intensity ]
                pass
            pass
        self.sorted = False
        return self

    def __get_integer( value ):
        if value == None: return 0
        tp = type( value )
        if tp == types.IntType: return value
        if tp == types.FloatType: return int( value )
        if tp == types.StringType:
            try:
                return string.atoi( value )
            except:
                return 0
            pass
        if tp == types.UnicodeType:
            return EarthQuakeInformation.getint( value.encode( 'utf-8' ) )
        return EarthQuakeInformation.getint( repr( value ) )

    def __get_float( value ):
        if value == None: return 0.0
        tp = type( value )
        if tp == types.IntType: return float( value )
        if tp == types.FloatType: return value
        if tp == types.StringType:
            try:
                return string.atof( value )
            except:
                return 0.0
            pass
        if tp == types.UnicodeType:
            return EarthQuakeInformation.getint( value.encode( 'utf-8' ) )
        return EarthQuakeInformation.getfloat( repr( value ) )

    getint = staticmethod( __get_integer )
    getfloat = staticmethod( __get_float )

    def set_report_date( self, year, month, date, hour, minute ):
        self.report_date = ( EarthQuakeInformation.getint( year ),
                             EarthQuakeInformation.getint( month ),
                             EarthQuakeInformation.getint( date ),
                             EarthQuakeInformation.getint( hour ),
                             EarthQuakeInformation.getint( minute ) )
        pass

    def set_date( self, day, hour, minute ):
        rdate = self.report_date
        ( ryear, rmonth, rday ) = ( rdate[ 0 ], rdate[ 1 ], rdate[ 2 ] )
        year, month = ryear, rmonth
        day, hour, minute = EarthQuakeInformation.getint( day ), EarthQuakeInformation.getint( hour ), EarthQuakeInformation.getint( minute )
        if day > rday: month -= 1
        if month <= 0:
            month = 12
            year -= 1
            pass
        self.incident_date = ( year, month, day, hour, minute )
        pass

    def set_latitude( self, latitude ):
        self.latitude = EarthQuakeInformation.getfloat( latitude )
        pass

    def set_longitude( self, longitude ):
        self.longitude = EarthQuakeInformation.getfloat( longitude )
        pass

    def set_depth( self, depth ):
        self.depth = EarthQuakeInformation.getint( depth )
        pass

    def set_magnitude( self, magnitude ):
        self.magnitude = EarthQuakeInformation.getfloat( magnitude )
        pass

    def set_links( self, url, prev, post ):
        self.url = url
        self.url_prev = prev
        self.url_post = post
        pass

    def sort_list( self ):
        if self.sorted: return
        self.arealist = self.qlist.keys()
        def comp( key1, key2 ):
            # sort by prefecture
            p1 = key1[ 0: 2 ]
            p2 = key2[ 0: 2 ]
            if p1 < p2: return -1
            if p1 > p2: return 0
            # sort by intensity
            d1 = self.qlist[ key1 ]
            d2 = self.qlist[ key2 ]
            p1 = d1[ 0 ] * d2[ 1 ]
            p2 = d1[ 1 ] * d2[ 0 ]
            if p1 < p2: return -1
            return 0
        self.arealist.sort( comp )
        self.sorted = True
        pass
    
    def __repr__( self ):
        output = ""
        rd = self.report_date
        if self.url:
            output += "URL      : %s\n" % ( self.url )
            pass
        if self.url_prev:
            output += "Previous : %s\n" % ( self.url_prev )
            pass
        if self.url_post:
            output += "Next     : %s\n" % ( self.url_post )
            pass
        output += "Report   : %4d/%2d/%2d/%2d/%2d\n" % ( rd[ 0 ], rd[ 1 ], rd[ 2 ], rd[ 3 ], rd[ 4 ] )
        rd = self.incident_date
        output += "Incident : %4d/%2d/%2d/%2d/%2d\n" % ( rd[ 0 ], rd[ 1 ], rd[ 2 ], rd[ 3 ], rd[ 4 ] )
        output += "Depth    : %d\n" % ( self.depth )
        output += "Latitude : %.1f\n" % ( self.latitude )
        output += "Longitude: %.1f\n" % ( self.longitude )
        #output += "Position : ( Latitude %4.1f/ Longitude %5.1f )\n" % ( self.latitude, self.longitude )
        output += "Magnitude: %.1f\n" % ( self.magnitude )
        output += "Max      : %d\n" % ( int( self.max_intensity ) )

        self.sort_list()

        for key in self.arealist:
            data = self.qlist[ key ]
            area = key
            #area = monitoringPoints.convert_code_to_name( key )
            intensity = int( float( data[ 1 ] ) / data[ 0 ] + .5 )
            output += "%s : %d\n" % ( area, intensity )
        return output

    def read_basic_information( self, elem ):
        children = elem.get_children()

        re_report = re.compile( u'発表$' )
        re_rdate = re.compile( u'(\\d+)年(\\d+)月(\\d+)日(\\d+)時(\\d+)分' )
        re_date = re.compile( u'(\\d+)日(\\d+)時(\\d+)分' )
        re_latitude = re.compile( u'北緯([\\d]+(\\.\\d)?)' )
        re_longitude = re.compile( u'東経([\\d]+(\\.\\d)?)' )
        re_magnitude = re.compile( u'地震の規模[^\\d]*(\\d+(\\.\\d+)?)' )
        re_depth = re.compile( u'深さは約?(\\d+(\\.\\d+)?)km' )
        info = EarthQuakeInformation()

        for child in children:
            if isinstance( child, Text ) == False: continue
            text = child.get_text().strip()
            utext = unicode( text, 'utf-8' )
            if re_report.search( utext ):
                m = re_rdate.search( utext )
                if m:
                    nums = m.groups()
                    self.set_report_date( nums[ 0 ], nums[ 1 ], nums[ 2 ], nums[ 3 ], nums[ 4 ] )
                    pass
                continue
            matcher = re_date.search( utext )
            if matcher:
                nums = matcher.groups()
                self.set_date( nums[ 0 ], nums[ 1 ], nums[ 2 ] )
                continue
            matcher = re_latitude.search( utext )
            if matcher:
                lat = string.atof( matcher.group( 1 ).encode( 'utf-8' ) )
                self.set_latitude( lat )
                pass
            matcher = re_longitude.search( utext )
            if matcher:
                lgt = string.atof( matcher.group( 1 ).encode( 'utf-8' ) )
                self.set_longitude( lgt )
                pass
            matcher = re_magnitude.search( utext )
            if matcher:
                mgn = string.atof( matcher.group( 1 ).encode( 'utf-8' ) )
                self.set_magnitude( mgn )
                pass
            matcher = re_depth.search( utext )
            if matcher:
                depth = string.atof( matcher.group( 1 ).encode( 'utf-8' ) )
                self.set_depth( depth )
                pass
            
            #print "#", child
            pass
        pass

    def add_area( self, prefecture, intensity, area ):
        import LocationData
        self.sorted = False
        if intensity > self.max_intensity:
            self.max_intensity = intensity
            pass
        code = LocationData.monitoringPoints.resolve_code( prefecture, area )
        if code:
            if self.qlist.has_key( code ):
                data_current = self.qlist[ code ]
                data_current[ 0 ] += 1
                data_current[ 1 ] += intensity
            else:
                self.qlist[ code ] = [ 1, intensity ]
                pass
            #print code, intensity
            pass
        else:
            #print "no code for ", area
            pass
        return

    def set_area_data( self, elem ):
        prefecture = None
        intensity = None
        areas = None
        itvalue = 0
        re_intensity = re.compile( u'震度([１２３４５６７８])' )
        re_area = re.compile( u'(([^市郡区町村]+[市(支庁)郡区町村]+).*)' )
        while elem:
            items = elem.get_text( "/" ).split( "/" )
            if len( items[ 0 ] ) > 0: prefecture = items[ 0 ]
            if len( items[ 1 ] ) > 0: intensity = items[ 1 ]
            #print intensity
            #print items[ 0 ], items[ 1 ], items[ 2 ]
            matcher = re_intensity.match( unicode( intensity, 'utf-8' ) )
            if matcher == None:
                itst = None
                itvalue = 0
            else:
                itst = matcher.group( 1 )
                pass
            if itst == u'１': itvalue = 1
            elif itst == u'２': itvalue = 2
            elif itst == u'３': itvalue = 3
            elif itst == u'４': itvalue = 4
            elif itst == u'５': itvalue = 5
            elif itst == u'６': itvalue = 6
            elif itst == u'７': itvalue = 7
            else: itst = None
            
            if itvalue > 0:
                for area in items[ 2: ]:
                    m = re_area.match( unicode( area, 'utf-8' ) )
                    if m:
                        areaname = m.group( 2 ).encode( 'utf-8' )
                        #print m.group( 2 ).encode( 'utf-8' )
                        self.add_area( prefecture, itvalue, areaname )
                        pass
                    #else:
                    #    print area
                    pass
                pass
            try:
                elem = elem.next_sibling( "tr" )
            except:
                break
            pass
        pass

    pass

def get_db_filename( directory, url ):
    import os.path
    if directory == None or os.path.isdir( directory ) == False:
        return None
    #print url
    m = re.match( "http://.*?/([^/]*)\\.html", url )
    if m:
	#print "MATCH"
        if directory[ -1 ] == '/': directory = directory[ 0: -1 ]
        filename = "%s/%s.eqdb" % ( directory, m.group( 1 ) )
	#print filename
        return filename
    else:
	return None
	#print "UNMATCH"
	#print url
    return None



def get_earthquake_file( directory, url ):
    unlinkflag = False
    urlbase = "http://www.jma.go.jp/jp/quake"
    if url.find( "http://" ) == 0: #load
        filename_db = get_db_filename( dir_output, url )
        # show cached filename
        if filename_db and os.path.exists( filename_db ):
            return ( filename_db, None )
        if url[ -5: ] != ".html": # current
            url = "http:///latestdata.html"
            pass
        # load from internet
        import tempfile
        filename_temp = tempfile.mktemp()
        lp = url.rfind( '/' )
        if lp >= 0: url_base = url[ 0: lp ]
        os.system( "curl \"%s\" -o \"%s\" -s" % ( arg, filename_temp ) )
        filename = tempfile.mktemp( "utf8" )
        os.system( "iconv -f sjis -t utf-8 %s > %s" % ( filename_temp, filename ) )
        os.unlink( filename_temp )
        unlinkflag = True
        pass
    else:
        filename = url
        url = "http:///test.html"
        pass    

    data = ""
    p = DOMConverter()
    p.add_ignoring_tag( "colgroup" )
    p.add_ignoring_tag( "col" )
    p.add_ignoring_tag( "div" )
    p.add_ignoring_tag( "li" )

    url_prev = None
    url_next = None
    re_urlJump = re.compile( "urlJump\('([^']+)'\)" )

    # read document and links
    for line in open( filename ).readlines():
        line = line.strip()
	#print line
        if line.find( "＜前の情報" ) >= 0:
            m = re_urlJump.search( line )            
            if m: url_prev = "%s/%s.html" % ( urlbase, m.group( 1 ) )
            pass
        if line.find( "次の情報＞" ) >= 0:
            m = re_urlJump.search( line )
            if m: url_next = "%s/%s.html" % ( urlbase, m.group( 1 ) )
            pass
        p.feed( line )
        pass
    if unlinkflag: os.unlink( filename )
    doc = p.get_document()

    # load earthquake information
    info = EarthQuakeInformation()
    info.set_links( url, url_prev, url_next )
    #print doc
    elem_base = None
    for i in range( 1, 5 ):
        elem_base = doc.get_element_at_path( "/html/body/table,%d/tr/td" % i )
        if elem_base != None and len( elem_base.get_children() ) > 1:
            break
        pass
    #print elem_base
    #print elem_base.get_element_at_path( "table/tr,1" )
    info.read_basic_information( elem_base )#doc.get_element_at_path( "/html/body/table,4/tr/td" ) )
    try:
	info.set_area_data( elem_base.get_element_at_path( "table" ) )
	pass
    except:
	info.set_area_data( elem_base.get_element_at_path( "table/tr" ) )
	pass

    #sys.exit()
    #info.read_basic_information( doc.get_element_at_path( "/html/body/table,4/tr/td" ) )
    #info.set_area_data( doc.get_element_at_path( "/html/body/table,4/tr/td/table/tr,2" ) )

    # output to file
    filename_output = get_db_filename( dir_output, url )
    if filename_output == None:
        filename_output = get_db_filename( "./", url )
        pass
    if filename_output:
	#print "OUT ", filename_output
        if info.is_linked() == False:
            filename_output = get_db_filename( dir_output, "http:///unlinked.html" );
	    #print "NEW NAME ", filename_output
            pass
	#print "OUT2 ", filename_output
        file_out = open( filename_output, "w" )
        file_out.write( repr( info ) )
        file_out.close()
        pass
    else:
        print info
        pass
    return ( filename_output, info )


if __name__ == "__main__":
    args = sys.argv
    args.reverse()
    #args = sys.argv.reverse()
    scriptname = args.pop()
    arg = args.pop()
    dir_output = None
    while len( args ) > 0:
        value = args.pop()
        if value == "-d": # output directory
            dir_output = args.pop()
            pass
        pass
    ( filename_db, obj ) = get_earthquake_file( dir_output, arg )
    print filename_db
    #if obj == None:
    #    obj = EarthQuakeInformation()
    #    obj.load_from_file( filename_db )
    #    pass
    #print obj
    pass

